using System;
using System.IO;
using System.ComponentModel;
using System.Text;

namespace id.Quake.Files {

	public partial class Model {

		/// <summary>Represents a model's skin.</summary>
		[TypeConverter(typeof(ExpandableObjectConverter))]
		public class Skin {

			#region Types

			/// <summary>Represents a single texture frame.</summary>
			[TypeConverter(typeof(ExpandableObjectConverter))]
			public class Texture : IPicture {

				#region Properties

				public int Width {
					get { return this.Skin.Width; }
				}
				public int Height {
					get { return this.Skin.Height; }
				}

				public override string ToString() {
					return string.Format("{0}x{1}", this.Width, this.Height);
				}

				private readonly float time;
				/// <summary>Gets the time offset of the texture.</summary>
				public float Time {
					get { return this.time; }
				}

				public Picture Picture {
					get {
						Palette P;
						if (!this.Skin.Model.Resources.TryGetCachedObject("gfx/palette.lmp", out P)) {
							P = this.Skin.Model.Resources.LoadObject<Palette>("gfx/palette.lmp");
							this.Skin.Model.Resources.CacheObject("gfx/palette.lmp", P);
						}
						return new Picture(this.Width, this.Height, Array.ConvertAll<byte, int>(this.Data, delegate(byte b) { return P.Colours[b]; }));
					}
				}

				#endregion

				#region Private Members

				private readonly Skin Skin;

				/// <summary>The raw unformatted pixel data.</summary>
				private readonly byte[] Data;

				#endregion

				#region Constructor

				public Texture(Skin skin, Stream stream, bool timedSkins) {
					this.Skin = skin;
					BinaryReader TextureReader = new BinaryReader(stream);
					if (timedSkins) {
						this.time = TextureReader.ReadSingle();
					} else {
						this.time = float.NaN;
					}
					this.Data = TextureReader.ReadBytes(this.Width * this.Height);
				}

				#endregion
			}

			#endregion

			#region Properties

			public int Width {
				get { return this.Model.SkinWidth; }
			}
			public int Height {
				get { return this.Model.SkinHeight; }
			}

			/// <summary>The textures associated with a particular model.</summary>
			private readonly Texture[] textures;
			public Texture[] Textures {
				get { return this.textures; }
			}

			public override string ToString() {
				return string.Format("{2} {0}x{1} texture{3}", this.Width, this.Height, this.Textures.Length, this.Textures.Length == 1 ? "" : "s");
			}

			#endregion

			#region Private Members

			private readonly Model Model;

			#endregion

			#region Constructor

			public Skin(Model model, Stream stream) {
				this.Model = model;

				BinaryReader SkinReader = new BinaryReader(stream);


				int TextureCount = SkinReader.ReadInt32();
				bool TimedSkins = false;
				if (TextureCount == 0) {
					TextureCount = 1;
				} else {
					TextureCount = SkinReader.ReadInt32();
					TimedSkins = true;
				}

				this.textures = new Texture[TextureCount];
				for (int i = 0; i < TextureCount; ++i) {
					this.textures[i] = new Texture(this, stream, TimedSkins);
				}
			}

			#endregion

			
		}
	}
}
