using System;
using System.IO;
using System.ComponentModel;
using System.Text;

namespace id.Quake.Files {

	public partial class Model {

		/// <summary>Represents a triangle.</summary>
		[TypeConverter(typeof(ExpandableObjectConverter))]
		public class Triangle {

			#region Properties

			private readonly bool facesFront;
			/// <summary>Specifies whether the triangle is on the front or back of the model.</summary>
			public bool FacesFront { get { return this.facesFront; } }

			private readonly int[] vertices;
			/// <summary>The three vertex indices this triangle is connected to.</summary>
			public int[] Vertices { get { return this.vertices; } }

			public override string ToString() {
				return string.Format("{1}{0}{2}{0}{3}", FacesFront ? "->" : "<-", Vertices[0], Vertices[1], Vertices[2]);
			}

			#endregion

			#region Extended Properties

			/// <summary>Gets the texture coordinates for the three corners of the triangle.</summary>
			public Vector2[] TextureCoordinates {
				get {
					Vector2[] Result = new Vector2[3];
					for (int i = 0; i < Result.Length; ++i) {
						TextureCoordinate TC = this.Model.TextureCoordinates[this.Vertices[i]];
						Result[i] = new Vector2(TC.X, TC.Y) / new Vector2(this.Model.SkinWidth, this.Model.SkinHeight);
						if (TC.IsOnSeam && !this.FacesFront) Result[i].X += 0.5f;
					}
					return Result;
				}
			}

			#endregion

			#region Private Members

			private readonly Model Model;

			#endregion

			#region Constructor

			public Triangle(Model model, Stream stream) {
				this.Model = model;
				BinaryReader TriangeReader = new BinaryReader(stream);
				this.facesFront = TriangeReader.ReadUInt32() != 0;
				this.vertices = new int[] { TriangeReader.ReadInt32(), TriangeReader.ReadInt32(), TriangeReader.ReadInt32() };
			}

			#endregion






		}
	}
}
