using System;
using System.Collections.Generic;
using QuakeC = id.Quake.Files.QuakeC;
using System.IO;

namespace id.Quake {
	public partial class VirtualMachine {

		private Dictionary<string, int> ModelCacheIndex;
		private List<object> ModelCache;

		private int CacheModel(string model) {
			int Index = 0;
			if (this.ModelCacheIndex.TryGetValue(model, out Index)) return Index;
			Index = ModelCache.Count;

			if (model[0] == '*') {
				this.ModelCache.Add(this.level.Models[int.Parse(model.Substring(1))]);
			} else {
				switch (Path.GetExtension(model)) {
					case ".bsp":
						this.ModelCache.Add(this.Resources.LoadObject<Files.Level>(model));
						break;
					case ".mdl":
						this.ModelCache.Add(this.Resources.LoadObject<Files.Model>(model));
						break;
					case ".spr":
						this.ModelCache.Add(null);
						break;
				}
			}
			this.ModelCacheIndex.Add(model, Index);
			return Index;
		}

		/// <summary>Get cached models.</summary>
		/// <typeparam name="T">Type type of the models to retrieve.</typeparam>
		public T[] GetCachedModels<T>() {
			List<T> Results = new List<T>(this.ModelCache.Count);
			foreach (object o in this.ModelCache) {
				if (o != null && o.GetType() == typeof(T)) Results.Add((T)o);
			}
			return Results.ToArray();
		}
	}
}
