using System;
using QuakeC = id.Quake.Files.QuakeC;

namespace id.Quake {
	public partial class VirtualMachine {

		/// <summary>Calculates the intersection point between a start position, end position, and the world.</summary>
		/// <param name="start">The position to start from.</param>
		/// <param name="end">The position to try to end at.</param>
		/// <returns>Information about the collision.</returns>
		public CollisionResult<Vector3> GetCollision(Vector3 start, Vector3 end) {
			return this.level.GetCollision(start, end);
		}

		/// <summary>Get the collision position between a bounding box and a face.</summary>
		/// <param name="box">The bounding box to move.</param>
		/// <param name="direction">The direction vector to move the box by.</param>
		public CollisionResult<BoundingBox> GetCollision(BoundingBox box, Vector3 direction) {

			BoundingBox End = box + direction;

			CollisionResult<BoundingBox> Result = new CollisionResult<BoundingBox>(false, End);

			foreach (Files.Level.Face F in this.level.Faces) {
				CollisionResult<BoundingBox> C = F.GetCollision(box, direction);
				if (C.CollisionOccured) {
					if (!Result.CollisionOccured || C.Distance < Result.Distance) {
						Result = C;
					}
				}
			}
			return Result;
		}
	
	}
}
